# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - 审计中心 (BlueKing - Audit Center) available.
Copyright (C) 2022 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import os
from dataclasses import dataclass, field
from typing import List

from django.conf import settings as django_settings
from django.utils.module_loading import import_string

from bk_audit.constants.contrib import DJANGO_SETTING_NAME
from bk_audit.constants.log import DEFAULT_QUEUE_LIMIT


@dataclass
class BKAuditSetting:
    """
    审计 SDK 配置
    """

    log_queue_limit: int = DEFAULT_QUEUE_LIMIT
    formatter: str = "bk_audit.log.formatters.Formatter"
    exporters: List[str] = field(
        default_factory=lambda: ["bk_audit.contrib.opentelemetry.exporters.OTLogExporter"]
        if os.getenv("BKAPP_OTEL_LOG_ENDPOINT")
        else ["bk_audit.log.exporters.LoggerExporter"]
    )
    service_name_handler: str = "bk_audit.contrib.opentelemetry.utils.ServiceNameHandler"
    ot_endpoint: str = ""
    bk_data_id: str = ""
    bk_data_token: str = ""

    def __post_init__(self):
        self.formatter = import_string(self.formatter)()
        self.exporters = [import_string(exporter)() for exporter in self.exporters]
        self.service_name_handler = import_string(self.service_name_handler)

    def get_app_code(self) -> str:
        return django_settings.APP_CODE

    def get_app_secret(self) -> str:
        return django_settings.SECRET_KEY


bk_audit_settings = BKAuditSetting(**getattr(django_settings, DJANGO_SETTING_NAME, {}))



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - 审计中心 (BlueKing - Audit Center) available.
Copyright (C) 2022 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import warnings


class BaseQueue(object):
    """队列"""

    def __init__(self, max_limit=0):
        """
        @type max_limit: int
        @param max_limit: 队列最大长度
        @rtype: BaseQueue
        """
        self._max_limit = max_limit
        self._storage = list()

    def _check_limit(self):
        """
        检查是否超过最大长度
        """
        current = len(self._storage)
        if self._max_limit and current > self._max_limit:
            item = self.pop()
            warnings.warn("Exceed Queue Limit %d(>%d), Drop Item %s" % (current, self._max_limit, str(item)))
            self._check_limit()

    def set_limit(self, limit):
        """
        设置最大长度
        @type limit: int
        @param limit: 队列最大长度
        """
        assert isinstance(limit, int), "Limit Invalid (Not A Number)"
        self._max_limit = limit
        self._check_limit()

    def add(self, item):
        """
        添加
        @type item: object
        @param item: 对象
        """
        self._storage.append(item)
        self._check_limit()

    def bulk_add(self, items):
        """
        批量添加
        @type items: typing.List[object]
        @param items: 对象列表
        """
        self._storage.extend(items)
        self._check_limit()

    def pop(self):
        """
        获取单个
        @rtype: typing.List[object]
        """
        if self._storage:
            return [self._storage.pop(0)]
        return []

    def pop_all(self):
        """
        获取所有
        @rtype: typing.List[object]
        """
        items = list()
        flag = True
        while flag:
            _items = self.pop()
            if _items:
                items.extend(_items)
                continue
            flag = False
        return items

    def clear(self):
        """
        清空队列
        """
        self._storage = list()


class AuditEventQueue(BaseQueue):
    """审计事件队列"""

    pass



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - 审计中心 (BlueKing - Audit Center) available.
Copyright (C) 2022 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import abc
import logging

from bk_audit.constants.utils import LOGGER_NAME


class BaseExporter(object):
    """
    审计事件导出
    """

    @property
    @abc.abstractmethod
    def is_delay(self):
        """
        True: 日志输入后直接导出
        False: 日志输入后，等待调用 BkAuditLog.export_events 时导出
        @rtype: bool
        """
        pass

    @abc.abstractmethod
    def export(self, events):
        """
        实际输出方法
        @type events: typing.List[bk_audit.log.models.AuditEvent]
        @param events: 审计事件列表
        """
        pass


class LoggerExporter(BaseExporter):
    """
    日志导出
    """

    is_delay = False

    def __init__(self):
        self.logger = logging.getLogger(LOGGER_NAME)
        self.logger.setLevel(logging.INFO)
        self.logger.propagate = False

    def export(self, events):
        """
        直接输出到日志
        @type events: typing.List[bk_audit.log.models.AuditEvent]
        @param events: 审计事件列表
        """
        for event in events:
            self.logger.info(event.to_json_str())



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - 审计中心 (BlueKing - Audit Center) available.
Copyright (C) 2022 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import logging
import uuid
from unittest import TestCase

from django.conf import settings
from django.core.mail.backends.locmem import EmailBackend
from django.test import override_settings

from bk_audit.constants.contrib import LoggingDefaultConfig
from bk_audit.constants.utils import LOGGER_NAME, OT_LOGGER_NAME
from bk_audit.contrib.django.formatters import DjangoFormatter
from bk_audit.contrib.django.loggers import LoggingConfigHandler
from bk_audit.log.models import AuditContext
from tests.base.client import init_client
from tests.base.constants import APP_CODE, REQUEST_IP, VIEW_FILE
from tests.base.models import Request


class TestDjango(TestCase):
    """测试Django集成"""

    def setUp(self):
        """初始化 Client"""
        self.client = init_client()
        self.client.set_formatter(DjangoFormatter())
        EmailBackend()

    def test_add_event(self):
        """测试添加事件"""
        # 常规事件
        request = Request()
        context = AuditContext(request=request)
        self.client.add_event(action=VIEW_FILE, audit_context=context)
        # APIGW事件
        setattr(request, "app", object())
        self.client.add_event(action=VIEW_FILE, audit_context=context)

    def test_meta_ip(self):
        """测试META IP"""
        # 常规Request
        request = Request()
        request.META = {}
        self.client.add_event(action=VIEW_FILE, audit_context=AuditContext(request=request))
        # HTTP_X_REAL_IP
        request.META = {"HTTP_X_REAL_IP": REQUEST_IP}
        self.client.add_event(action=VIEW_FILE, audit_context=AuditContext(request=request))
        # HTTP_X_FORWARDED_FOR
        request.META = {"HTTP_X_FORWARDED_FOR": REQUEST_IP}
        self.client.add_event(action=VIEW_FILE, audit_context=AuditContext(request=request))

    def test_logging(self):
        """测试LOGGING"""
        file_name = "test.log"
        logging_config = LoggingConfigHandler(file_name).set_logging({"handlers": {}, "formatters": {}, "loggers": {}})
        self.assertEqual(
            logging_config,
            {
                "handlers": {
                    "bk_audit": {
                        "class": LoggingDefaultConfig.HANDLER_CLS,
                        "formatter": LOGGER_NAME,
                        "filename": file_name,
                        "maxBytes": LoggingDefaultConfig.FILE_MAX_BYTES,
                        "backupCount": LoggingDefaultConfig.FILE_BACKUP_COUNT,
                    }
                },
                "formatters": {LOGGER_NAME: {"format": "%(message)s"}},
                "loggers": {LOGGER_NAME: {"handlers": [LOGGER_NAME], "level": logging.INFO, "propagate": True}},
            },
        )

    def test_auto_instrument(self):
        """测试自动注入"""

        settings.configure(
            APP_CODE=APP_CODE, SECRET_KEY=uuid.uuid1().hex, BK_AUDIT_SETTINGS={"ot_endpoint": "http://127.0.0.1"}
        )

        from bk_audit.contrib.bk_audit.apps import AuditConfig

        app_config = AuditConfig.create("bk_audit.contrib.bk_audit")
        app_config.ready()

        # 清理 Logger 避免影响其他的单元测试
        logging.getLogger(OT_LOGGER_NAME).handlers = []

    @override_settings()
    def test_auto_instrument_ot(self):
        """测试OT注入"""

        from bk_audit.contrib.bk_audit.settings import bk_audit_settings

        bk_audit_settings.ot_endpoint = ""

        from bk_audit.contrib.bk_audit.apps import AuditConfig

        app_config = AuditConfig.create("bk_audit.contrib.bk_audit")
        app_config.ready()

        # 清理 Logger 避免影响其他的单元测试
        logging.getLogger(OT_LOGGER_NAME).handlers = []

    @override_settings()
    def test_resource(self):
        """测试调用Resource"""

        # 初始化
        from bk_audit.contrib.bk_audit.apps import AuditConfig

        app_config = AuditConfig.create("bk_audit.contrib.bk_audit")
        app_config.ready()

        # 更新格式化器
        from bk_audit.contrib.bk_audit.client import bk_audit_client

        bk_audit_client.set_formatter(DjangoFormatter())

        # 测试正常、异常、标准异常、未实现三种状态

        from tests.base.resources import (
            BlueErrorResource,
            ErrorResource,
            NoAuditResource,
            Resource,
        )

        NoAuditResource().request()
        Resource().request()
        with self.assertRaises(Exception):
            ErrorResource().request()
        with self.assertRaises(Exception):
            BlueErrorResource().request()



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - 审计中心 (BlueKing - Audit Center) available.
Copyright (C) 2022 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import logging
from functools import wraps

from bk_audit.constants.utils import LOGGER_NAME


def ignore_wrapper(message=False, default_return=None):
    """
    忽略运行时的错误，返回默认值
    @type message: bool
    @param message: 是否输出错误日志
    @type default_return: any
    @param default_return: 错误时的默认返回值
    """

    def ignore(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            try:
                return func(*args, **kwargs)
            except Exception as err:  # pylint: disable=broad-except
                if message:
                    logger = logging.getLogger(LOGGER_NAME)
                    logger.exception(err)
                return default_return

        return wrapper

    return ignore



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - 审计中心 (BlueKing - Audit Center) available.
Copyright (C) 2022 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import logging
import os
from typing import Type

from opentelemetry._logs import set_logger_provider
from opentelemetry.exporter.otlp.proto.grpc._log_exporter import OTLPLogExporter
from opentelemetry.sdk._logs import LoggerProvider, LoggingHandler, LogRecordProcessor
from opentelemetry.sdk._logs.export import SimpleLogRecordProcessor
from opentelemetry.sdk.resources import Resource

from bk_audit.constants.utils import OT_LOGGER_NAME
from bk_audit.contrib.opentelemetry.processor import LazyBatchLogProcessor


def setup(
    client,
    bk_data_id=None,
    bk_data_token=None,
    endpoint=None,
):
    """
    初始化OT日志
    @type client: bk_audit.client.BkAudit
    @param client: 审计Client
    @type bk_data_id: int
    @param bk_data_id: Data ID
    @type bk_data_token: str
    @param bk_data_token: Data Token
    @type endpoint: str
    @param endpoint: 上报地址
    """

    # init service config
    service_name = client.service_name
    bk_data_id = bk_data_id or int(os.getenv("BKAPP_OTEL_LOG_BK_DATA_ID", -1))
    bk_data_token = bk_data_token or os.getenv("BKAPP_OTEL_LOG_BK_DATA_TOKEN", "")

    # init log emitter
    logger_provider = LoggerProvider(
        resource=Resource.create(
            {
                "service.name": service_name,
                "bk_data_id": bk_data_id,
                "bk.data.token": bk_data_token,
            }
        )
    )
    set_logger_provider(logger_provider)

    # init exporter
    processor: Type[LogRecordProcessor] = LazyBatchLogProcessor
    if os.getenv("BKAPP_USE_SIMPLE_LOG_PROCESSOR"):
        processor = SimpleLogRecordProcessor
    exporter = OTLPLogExporter(endpoint=endpoint or os.getenv("BKAPP_OTEL_LOG_ENDPOINT"))
    logger_provider.add_log_record_processor(processor(exporter))

    # init logging
    handler = LoggingHandler(level=logging.NOTSET, logger_provider=logger_provider)
    logging.getLogger(OT_LOGGER_NAME).addHandler(handler)



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - 审计中心 (BlueKing - Audit Center) available.
Copyright (C) 2022 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

from unittest import TestCase
from unittest.mock import patch

from bk_audit.utils.decorators import ignore_wrapper
from bk_audit.utils.time_tool import get_current_ts
from tests.base.models import Datetime, Empty


class TestUtils(TestCase):
    """测试工具"""

    def test_ignore_wrapper(self):
        """测试忽略异常"""

        empty = Empty()

        @ignore_wrapper(default_return=empty, message=True)
        def func():
            raise Exception

        self.assertEqual(func(), empty)

    def test_get_current_ts(self):
        """测试获取时间戳"""
        self.assertEqual(type(get_current_ts()), float)

    @patch("bk_audit.utils.time_tool.datetime", Datetime())
    def test_get_current_ts_py2(self):
        """测试获取时间戳(Py2)"""
        self.assertEqual(type(get_current_ts()), float)



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - 审计中心 (BlueKing - Audit Center) available.
Copyright (C) 2022 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import os
from unittest import TestCase
from unittest.mock import Mock, patch

from opentelemetry.exporter.otlp.proto.grpc._log_exporter import OTLPLogExporter
from opentelemetry.sdk._logs import LogData, LogRecord
from opentelemetry.sdk.util.instrumentation import InstrumentationScope

from bk_audit.contrib.opentelemetry.exporters import OTLogExporter
from bk_audit.contrib.opentelemetry.processor import LazyBatchLogProcessor
from bk_audit.contrib.opentelemetry.setup import setup
from bk_audit.contrib.opentelemetry.utils import (
    BaseServiceNameHandler,
    ServiceNameHandler,
)
from tests.base.client import init_client
from tests.base.constants import CONTEXT, SERVICE_NAME, VIEW_FILE


class TestOT(TestCase):
    """测试Client"""

    def setUp(self):
        """初始化 Client"""
        self.client = init_client([OTLogExporter()])

    def test_ot_export(self):
        """测试OT导出"""
        setup(self.client)
        self.client.add_event(action=VIEW_FILE, audit_context=CONTEXT)

    def test_json_decode_error(self):
        """测试JSON转换失败"""
        result_content = object()
        self.client.add_event(action=VIEW_FILE, audit_context=CONTEXT, result_content={"content": result_content})

    def test_setup(self):
        """测试OT初始化"""
        setup(self.client)

    def test_service_name(self):
        """测试基类"""
        self.assertEqual(BaseServiceNameHandler(str()).get_service_name(), None)

    @patch("bk_audit.contrib.opentelemetry.utils.ServiceNameHandler.is_celery_beat", Mock(return_value=True))
    def test_service_name_beat(self):
        """测试服务名处理Beat"""
        self.assertEqual(ServiceNameHandler(SERVICE_NAME).get_service_name(), "%s_celery_beat" % SERVICE_NAME)

    @patch("bk_audit.contrib.opentelemetry.utils.ServiceNameHandler.is_celery", Mock(return_value=True))
    def test_service_name_worker(self):
        """测试服务名处理Worker"""
        self.assertEqual(ServiceNameHandler(SERVICE_NAME).get_service_name(), "%s_celery_worker" % SERVICE_NAME)

    def test_processor_shutdown(self):
        """测试停止进程"""
        exporter = OTLPLogExporter(endpoint=os.getenv("BKAPP_OTEL_LOG_ENDPOINT"))
        # shutdown normally
        processor = LazyBatchLogProcessor(exporter)
        processor.force_flush()
        processor.shutdown()
        # shutdown with work thread
        processor = LazyBatchLogProcessor(exporter)
        processor.emit(LogData(LogRecord(), InstrumentationScope(self.client._bk_app_code)))
        processor.force_flush()
        processor.shutdown()



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - 审计中心 (BlueKing - Audit Center) available.
Copyright (C) 2022 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import abc
import sys


class BaseServiceNameHandler(object):
    """
    服务名处理
    """

    def __init__(self, service_name):
        """
        @type service_name: str
        @param service_name: 服务名
        @rtype: BaseServiceNameHandler
        """
        self._service_name = service_name

    @abc.abstractmethod
    def get_service_name(self):
        """
        获取服务名
        @rtype: str
        """
        pass


class ServiceNameHandler(BaseServiceNameHandler):
    """
    服务名处理
    """

    class SuffixEnum(object):
        CELERY_BEAT = "celery_beat"
        CELERY_WORKER = "celery_worker"
        API = "api"
        NONE = ""

    @property
    def is_celery(self):
        """
        判断是否为 Celery Worker
        @rtype: bool
        """
        return "celery" in sys.argv

    @property
    def is_celery_beat(self):
        """
        判断是否为 Celery Beat
        @rtype: bool
        """
        return self.is_celery and "beat" in sys.argv

    @property
    def suffix(self):
        """
        获取服务名后缀
        @rtype: str
        """
        if self.is_celery_beat:
            return self.SuffixEnum.CELERY_BEAT
        if self.is_celery:
            return self.SuffixEnum.CELERY_WORKER
        return self.SuffixEnum.NONE

    def get_service_name(self):
        """
        获取服务名
        @rtype: str
        """
        if self.suffix:
            return "{}_{}".format(self._service_name, self.suffix)
        return self._service_name



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - 审计中心 (BlueKing - Audit Center) available.
Copyright (C) 2022 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import uuid

from bk_audit.constants.log import (
    DEFAULT_EMPTY_VALUE,
    DEFAULT_QUEUE_LIMIT,
    DEFAULT_RESULT_CODE,
)
from bk_audit.log.base import BkAuditLog
from bk_audit.log.exporters import LoggerExporter
from bk_audit.log.formatters import Formatter
from bk_audit.log.models import (
    AuditAction,
    AuditContext,
    AuditInstance,
    AuditResourceType,
)
from bk_audit.utils.time_tool import get_current_ms_ts


class BkAudit(object):
    """审计SDK"""

    def __init__(self, bk_app_code, bk_app_secret, settings=None):
        """
        @type bk_app_code: str
        @param bk_app_code: App Code
        @type bk_app_secret: str
        @param bk_app_secret: App Secret
        @type settings: dict
        @param settings: 设置
        @rtype: BkAudit
        """
        self._bk_app_code = bk_app_code
        self._bk_app_secret = bk_app_secret
        self._settings = settings or {}
        self._log = BkAuditLog(self._bk_app_code, self._bk_app_secret)
        self.service_name = bk_app_code
        self._init_settings()

    def _init_settings(self):
        """
        从设置初始化
        """
        # 初始化审计日志队列长度
        queue_limit = self._settings.get("log_queue_limit", DEFAULT_QUEUE_LIMIT)
        self.set_queue_limit(queue_limit)
        # 初始化审计日志处理实例
        formatter = self._settings.get("formatter", Formatter())
        self.set_formatter(formatter)
        # 初始化审计日志输出实例
        exporters = self._settings.get("exporters", [LoggerExporter()])
        for exporter in exporters:
            self.add_exporter(exporter)
        # 初始化OT日志
        service_name_handler = self._settings.get("service_name_handler")
        if service_name_handler:
            self.service_name = service_name_handler(self.service_name).get_service_name()

    def set_queue_limit(self, limit):
        """
        设置队列长度
        @type limit: int
        @param limit: 审计事件队列最大长度
        """
        self._log.set_queue_limit(limit)

    def set_log(self, log):
        """
        设置审计日志实例
        @type log: BkAuditLog
        @param log: 审计日志实例
        """
        self._log = log

    def set_formatter(self, formatter):
        """
        设置审计事件处理实例
        @type formatter: bk_audit.log.formatters.BaseFormatter
        @param formatter: 审计事件处理器
        """
        self._log.set_formatter(formatter)

    def add_exporter(self, exporter):
        """
        添加审计事件输出实例
        @type exporter: bk_audit.log.exporters.BaseExporter
        @param exporter: 审计事件输出器
        """
        self._log.add_exporter(exporter)

    def add_event(
        self,
        action,
        resource_type=None,
        instance=None,
        audit_context=None,
        event_id=None,
        event_content=DEFAULT_EMPTY_VALUE,
        start_time=None,
        end_time=None,
        result_code=DEFAULT_RESULT_CODE,
        result_content=DEFAULT_EMPTY_VALUE,
        extend_data=DEFAULT_EMPTY_VALUE,
    ):
        """
        添加审计事件
        @type audit_context: bk_audit.log.models.AuditContext
        @param audit_context: 审计上下文
        @type action: bk_audit.log.models.AuditAction
        @param action: 操作
        @type resource_type: bk_audit.log.models.AuditResourceType
        @param resource_type: 资源类型
        @type instance: bk_audit.log.models.AuditInstance
        @param instance: 审计实例对象
        @type event_id: str
        @param event_id: 事件ID
        @type event_content: str
        @param event_content: 事件描述
        @type start_time: int
        @param start_time: 事件开始时间
        @type end_time: int
        @param end_time: 事件结束时间
        @type result_code: int
        @param result_code: 操作结果
        @type result_content: str
        @param result_content: 操作结果描述
        @type extend_data: dict
        @param extend_data: 拓展信息
        """
        if not audit_context:
            audit_context = AuditContext()
        if not instance:
            instance = AuditInstance(object())
        action = AuditAction(action) if action else AuditAction(object())
        resource_type = AuditResourceType(resource_type) if resource_type else AuditResourceType(object())
        self._log.add_event(
            audit_context=audit_context,
            action=action,
            resource_type=resource_type,
            instance=instance,
            event_id=event_id or uuid.uuid1().hex,
            event_content=event_content,
            start_time=start_time or get_current_ms_ts(),
            end_time=end_time or get_current_ms_ts(),
            result_code=result_code,
            result_content=result_content,
            extend_data=extend_data,
        )

    def export_events(self):
        """
        导出审计事件
        """
        self._log.export_events()



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - 审计中心 (BlueKing - Audit Center) available.
Copyright (C) 2022 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import os
import uuid
from unittest import TestCase

from bk_audit.log.base import BkAuditLog
from bk_audit.log.exporters import BaseExporter, LoggerExporter
from bk_audit.log.formatters import BaseFormatter, Formatter
from bk_audit.log.models import AuditContext, AuditEvent
from tests.base.client import app_code, app_secret, init_client
from tests.base.constants import CONTEXT, HOST, HOST_INSTANCE, VIEW_FILE
from tests.base.exporters import DelayExporter


class TestClient(TestCase):
    """测试Client"""

    def setUp(self):
        """初始化 Client"""
        self.client = init_client()

    def test_add_event(self):
        """测试新增审计事件"""
        self.client.add_event(action=VIEW_FILE, audit_context=CONTEXT)
        self.client.export_events()
        with self.assertRaises(AssertionError):
            self.client.add_event(action=VIEW_FILE)

    def test_set_formatter(self):
        """测试设置事件处理器"""
        # 设置处理器
        formatter = BaseFormatter()
        self.client.set_formatter(formatter)
        # 判断处理器设置成功
        self.assertEqual(self.client._log._formatter, formatter)
        # 设置处理器
        formatter = Formatter()
        self.client.set_formatter(formatter)
        # 判断处理器设置成功
        self.assertEqual(self.client._log._formatter, formatter)

    def test_add_exporter(self):
        """测试添加事件输出"""
        # 添加输出
        self.client.add_exporter(LoggerExporter())
        self.client.add_exporter(DelayExporter())
        self.assertEqual(len(self.client._log._exporter_class), 4)

    def test_set_queue_limit(self):
        """测试设置队列长度"""
        # 清空队列
        self.client._log._queue.clear()
        # 添加数据
        self.client.add_event(action=VIEW_FILE, audit_context=CONTEXT)
        self.client.add_event(action=VIEW_FILE, audit_context=CONTEXT)
        # 判断队列长度
        self.assertEqual(len(self.client._log._queue._storage), 2)
        # 设置最大长度为 1
        self.client.set_queue_limit(1)
        # 判断队列长度
        self.assertEqual(len(self.client._log._queue._storage), 1)

    def test_set_log(self):
        """测试设置日志类"""
        self.client.set_log(BkAuditLog(app_code, app_secret))

    def test_bulk_add(self):
        """测试批量添加"""
        self.client = init_client()
        event = AuditEvent()
        self.client._log._queue.bulk_add([event])
        self.assertEqual(len(self.client._log._queue._storage), 1)

    def test_base_formatter(self):
        """测试基础事件处理"""
        formatter = BaseFormatter()
        formatter.build_event(
            action=VIEW_FILE,
            resource_type=HOST,
            audit_context=AuditContext(),
            instance=HOST_INSTANCE,
            event_id=uuid.uuid1().hex,
            event_content=str(),
            start_time=int(),
            end_time=int(),
            result_code=int(),
            result_content=str(),
            extend_data=dict(),
        )

    def test_base_exporter(self):
        """测试基础导出"""
        exporter = BaseExporter()
        exporter.export([])
        self.assertEqual(exporter.is_delay, None)

    def test_model_str(self):
        """测试ModelStr"""
        event_id = uuid.uuid1().hex
        self.assertEqual(str(AuditEvent(event_id=event_id)), event_id)

    def test_setup_simple(self):
        """测试生成同步上报"""
        os.environ["BKAPP_USE_SIMPLE_LOG_PROCESSOR"] = "True"
        client = init_client()
        client.add_event(action=VIEW_FILE, audit_context=CONTEXT)



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - 审计中心 (BlueKing - Audit Center) available.
Copyright (C) 2022 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import json
import logging

from bk_audit.constants.utils import OT_LOGGER_NAME
from bk_audit.log.exporters import BaseExporter


class OTLogExporter(BaseExporter):
    """
    OT日志输出
    """

    is_delay = False

    def __init__(self):
        self.logger = logging.getLogger(OT_LOGGER_NAME)
        self.logger.setLevel(logging.INFO)
        self.logger.propagate = False

    def _trans_json(self, data):
        for key, val in data.items():
            if isinstance(val, (dict, list)):
                try:
                    data[key] = json.dumps(val)
                except Exception:  # pylint: disable=broad-except
                    data[key] = str(val)
            elif isinstance(val, int):
                data[key] = val
            elif val is None:
                data[key] = str()
            else:
                data[key] = str(val)
        return data

    def export(self, events):
        """
        实际输出方法
        @type events: typing.List[bk_audit.log.models.AuditEvent]
        @param events: 审计事件列表
        """
        for event in events:
            self.logger.info(event.event_content, extra=self._trans_json(event.to_json()))



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - 审计中心 (BlueKing - Audit Center) available.
Copyright (C) 2022 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import logging

from bk_audit.constants.contrib import LoggingDefaultConfig
from bk_audit.constants.utils import LOGGER_NAME


class LoggingConfigHandler(object):
    """
    处理Logging配置
    """

    def __init__(
        self,
        filename,
        log_level=logging.INFO,
        handler_cls=LoggingDefaultConfig.HANDLER_CLS,
        file_max_bytes=LoggingDefaultConfig.FILE_MAX_BYTES,
        file_backup_count=LoggingDefaultConfig.FILE_BACKUP_COUNT,
    ):
        self.filename = filename
        self.log_level = log_level
        self.handler_cls = handler_cls
        self.file_max_bytes = file_max_bytes
        self.file_backup_count = file_backup_count

    @property
    def handler_name(self):
        return LOGGER_NAME

    @property
    def handler_config(self):
        return {
            "class": self.handler_cls,
            "formatter": self.formatter_name,
            "filename": self.filename,
            "maxBytes": self.file_max_bytes,
            "backupCount": self.file_backup_count,
        }

    @property
    def formatter_name(self):
        return LOGGER_NAME

    @property
    def formatter_config(self):
        return {"format": "%(message)s"}

    @property
    def logger_name(self):
        return LOGGER_NAME

    @property
    def logger_config(self):
        return {"handlers": [self.handler_name], "level": self.log_level, "propagate": True}

    def set_logging(self, logging_config):
        logging_config["formatters"][self.formatter_name] = self.formatter_config
        logging_config["handlers"][self.handler_name] = self.handler_config
        logging_config["loggers"][self.logger_name] = self.logger_config
        return logging_config



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - 审计中心 (BlueKing - Audit Center) available.
Copyright (C) 2022 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import abc

from bk_audit.log.models import AuditEvent


class BaseFormatter(object):
    """
    事件处理
    """

    @abc.abstractmethod
    def build_event(
        self,
        action,
        resource_type,
        audit_context,
        instance,
        event_id,
        event_content,
        start_time,
        end_time,
        result_code,
        result_content,
        extend_data,
    ):
        """
        构造审计事件
        @type action: bk_audit.log.models.AuditAction
        @param action: 操作
        @type resource_type: bk_audit.log.models.AuditResourceType
        @param resource_type: 资源类型
        @type audit_context: bk_audit.log.models.AuditContext
        @param audit_context: 审计上下文
        @type instance: bk_audit.log.models.AuditInstance
        @param instance: 审计实例对象
        @type event_id: str
        @param event_id: 事件ID
        @type event_content: str
        @param event_content: 事件描述
        @type start_time: int
        @param start_time: 事件开始时间
        @type end_time: int
        @param end_time: 事件结束时间
        @type result_code: int
        @param result_code: 操作结果
        @type result_content: str
        @param result_content: 操作结果描述
        @type extend_data: dict
        @param extend_data: 拓展信息
        @rtype: AuditEvent
        """
        pass


class Formatter(BaseFormatter):
    """
    基础处理，直接转换为审计事件
    """

    def build_event(
        self,
        action,
        resource_type,
        audit_context,
        instance,
        event_id,
        event_content,
        start_time,
        end_time,
        result_code,
        result_content,
        extend_data,
    ):
        """
        构造审计事件
        @type action: bk_audit.log.models.AuditAction
        @param action: 操作
        @type resource_type: bk_audit.log.models.AuditResourceType
        @param resource_type: 资源类型
        @type audit_context: bk_audit.log.models.AuditContext
        @param audit_context: 审计上下文
        @type instance: bk_audit.log.models.AuditInstance
        @param instance: 审计实例对象
        @type event_id: str
        @param event_id: 事件ID
        @type event_content: str
        @param event_content: 事件描述
        @type start_time: int
        @param start_time: 事件开始时间
        @type end_time: int
        @param end_time: 事件结束时间
        @type result_code: int
        @param result_code: 操作结果
        @type result_content: str
        @param result_content: 操作结果描述
        @type extend_data: dict
        @param extend_data: 拓展信息
        @rtype: AuditEvent
        """
        return AuditEvent(
            event_id=event_id,
            event_content=event_content,
            request_id=audit_context.request_id,
            username=audit_context.username,
            user_identify_type=audit_context.user_identify_type,
            user_identify_tenant_id=audit_context.user_identify_tenant_id,
            user_identify_src=audit_context.user_identify_src,
            user_identify_src_username=audit_context.user_identify_src_username,
            start_time=start_time,
            end_time=end_time,
            scope_type=audit_context.scope_type,
            scope_id=audit_context.scope_id,
            access_type=audit_context.access_type,
            access_source_ip=audit_context.access_source_ip,
            access_user_agent=audit_context.access_user_agent,
            action_id=action.id,
            resource_type_id=resource_type.id,
            instance_id=instance.instance_id,
            instance_name=instance.instance_name,
            instance_sensitivity=instance.instance_sensitivity,
            instance_data=instance.instance_data,
            instance_origin_data=instance.instance_origin_data,
            result_code=result_code,
            result_content=result_content,
            extend_data=extend_data,
        )



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - 审计中心 (BlueKing - Audit Center) available.
Copyright (C) 2022 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import json
import uuid

from bk_audit.constants.log import (
    AUDIT_EVENT_SIGNATURE,
    DEFAULT_EMPTY_VALUE,
    DEFAULT_RESULT_CODE,
    DEFAULT_SENSITIVITY,
    AccessTypeEnum,
    UserIdentifyTypeEnum,
)
from bk_audit.utils.time_tool import get_current_ms_ts


class AuditEvent(object):
    """
    审计事件
    """

    def __init__(
        self,
        event_id=None,
        event_content=DEFAULT_EMPTY_VALUE,
        request_id=DEFAULT_EMPTY_VALUE,
        username=DEFAULT_EMPTY_VALUE,
        user_identify_type=UserIdentifyTypeEnum.UNKNOWN,
        user_identify_tenant_id=DEFAULT_EMPTY_VALUE,
        user_identify_src=DEFAULT_EMPTY_VALUE,
        user_identify_src_username=DEFAULT_EMPTY_VALUE,
        start_time=0,
        end_time=0,
        bk_app_code=DEFAULT_EMPTY_VALUE,
        scope_type=DEFAULT_EMPTY_VALUE,
        scope_id=DEFAULT_EMPTY_VALUE,
        access_type=AccessTypeEnum.OTHER,
        access_source_ip=DEFAULT_EMPTY_VALUE,
        access_user_agent=DEFAULT_EMPTY_VALUE,
        action_id=DEFAULT_EMPTY_VALUE,
        resource_type_id=DEFAULT_EMPTY_VALUE,
        instance_id=DEFAULT_EMPTY_VALUE,
        instance_name=DEFAULT_EMPTY_VALUE,
        instance_sensitivity=DEFAULT_SENSITIVITY,
        instance_data=DEFAULT_EMPTY_VALUE,
        instance_origin_data=DEFAULT_EMPTY_VALUE,
        result_code=DEFAULT_RESULT_CODE,
        result_content=DEFAULT_EMPTY_VALUE,
        extend_data=DEFAULT_EMPTY_VALUE,
    ):
        """
        @type event_id: str
        @param event_id: 事件ID
        @type event_content: str
        @param event_content: 事件描述
        @type request_id: str
        @param request_id: 请求ID
        @type username: str
        @param username: 操作人用户名
        @type user_identify_type: int
        @param user_identify_type: 操作人账号类型
        @type user_identify_tenant_id: str
        @param user_identify_tenant_id: 操作人租户ID
        @type user_identify_src: str
        @param user_identify_src: 操作人账号来源
        @type user_identify_src_username: str
        @param user_identify_src_username: 操作人账号
        @type start_time: int
        @param start_time: 事件开始事件
        @type end_time: int
        @param end_time: 事件结束时间
        @type bk_app_code: str
        @param bk_app_code: 事件上报模块
        @type scope_type: str
        @param scope_type: 管理空间类型
        @type scope_id: str
        @param scope_id: 管理空间ID
        @type access_type: int
        @param access_type: 访问方式
        @type access_source_ip: str
        @param access_source_ip: 访问来源IP地址
        @type access_user_agent: str
        @param access_user_agent: 访问客户端类型
        @type action_id: str
        @param action_id: 操作ID
        @type resource_type_id: str
        @param resource_type_id: 资源类型ID
        @type instance_id: str
        @param instance_id: 实例ID
        @type instance_name: str
        @param instance_name: 实例名称
        @type instance_sensitivity: int
        @param instance_sensitivity: 实例敏感等级
        @type instance_data: dict
        @param instance_data: 实例数据
        @type instance_origin_data: dict
        @param instance_origin_data: 实例原始数据
        @type result_code: int
        @param result_code: 操作结果
        @type result_content: str
        @param result_content: 操作结果描述
        @type extend_data: dict
        @param extend_data: 拓展信息
        @rtype: AuditEvent
        """
        self.event_id = event_id or uuid.uuid1().hex
        self.event_content = event_content
        self.request_id = request_id
        self.username = username
        self.user_identify_type = int(user_identify_type)
        self.user_identify_tenant_id = user_identify_tenant_id
        self.user_identify_src = user_identify_src
        self.user_identify_src_username = user_identify_src_username
        self.start_time = int(start_time) or get_current_ms_ts()
        self.end_time = int(end_time) or get_current_ms_ts()
        self.bk_app_code = bk_app_code
        self.scope_type = scope_type
        self.scope_id = scope_id
        self.access_type = int(access_type)
        self.access_source_ip = access_source_ip
        self.access_user_agent = access_user_agent
        self.action_id = action_id
        self.resource_type_id = resource_type_id
        self.instance_id = instance_id
        self.instance_name = instance_name
        self.instance_sensitivity = instance_sensitivity
        self.instance_data = instance_data or {}
        self.instance_origin_data = instance_origin_data or {}
        self.result_code = int(result_code)
        self.result_content = result_content
        self.extend_data = extend_data or {}
        self.bk_log_scope = AUDIT_EVENT_SIGNATURE

    def __str__(self):
        """
        字符串函数
        @rtype: str
        """
        return self.event_id

    def _validate(self):
        """
        数据校验
        """
        key_params = ["event_id", "username", "start_time", "action_id"]
        errors = []
        for key in key_params:
            if not getattr(self, key, None):
                errors.append(key)
        assert not errors, "Audit Event Key Params Unset => %s" % ",".join(errors)

    def to_json_str(self):
        """
        获取JSON字符串
        @rtype: str
        """
        return json.dumps(self.to_json(), ensure_ascii=False)

    def to_json(self):
        """
        获取JSON内容
        @rtype: dict
        """
        self._validate()
        return {key: val for key, val in self.__dict__.items() if not key.startswith("_") and not callable(val)}


class AuditInstance(object):
    """
    审计对象
    """

    def __init__(self, instance):
        """
        @type instance: object
        @param instance: 实例对象
        @rtype: AuditInstance
        """
        self.instance = instance

    @property
    def instance_id(self):
        """
        实例ID
        @rtype: str
        """
        return getattr(self.instance, "instance_id", DEFAULT_EMPTY_VALUE)

    @property
    def instance_name(self):
        """
        实例名
        @rtype: str
        """
        return getattr(self.instance, "instance_name", DEFAULT_EMPTY_VALUE)

    @property
    def instance_sensitivity(self):
        """
        实例风险等级
        @rtype: int
        """
        return getattr(self.instance, "instance_sensitivity", DEFAULT_SENSITIVITY)

    @property
    def instance_data(self):
        """
        实例信息 JSON
        @rtype: dict
        """
        return getattr(self.instance, "instance_data", DEFAULT_EMPTY_VALUE)

    @property
    def instance_origin_data(self):
        """
        实例修改前内容
        @rtype: dict
        """
        return getattr(self.instance, "instance_origin_data", DEFAULT_EMPTY_VALUE)


class AuditAction(object):
    """
    操作
    """

    def __init__(self, action):
        """
        @type action: iam.model.models.Action|object
        @param action: 审计操作
        @rtype: AuditAction
        """
        self.action = action

    @property
    def id(self):
        """
        操作ID
        @rtype: str
        """
        return getattr(self.action, "id", DEFAULT_EMPTY_VALUE)


class AuditResourceType(object):
    """
    资源类型
    """

    def __init__(self, resource_type):
        """
        @type resource_type: iam.model.models.ResourceType|object
        @param resource_type: 审计资源类型
        @rtype: AuditResourceType
        """
        self.resource_type = resource_type

    @property
    def id(self):
        """
        资源类型ID
        @rtype: str
        """
        return getattr(self.resource_type, "id", DEFAULT_EMPTY_VALUE)


class AuditContext(object):
    """
    上下文
    """

    def __init__(
        self,
        username=DEFAULT_EMPTY_VALUE,
        request_id=DEFAULT_EMPTY_VALUE,
        access_type=AccessTypeEnum.OTHER,
        access_source_ip=DEFAULT_EMPTY_VALUE,
        access_user_agent=DEFAULT_EMPTY_VALUE,
        user_identify_type=UserIdentifyTypeEnum.UNKNOWN,
        user_identify_tenant_id=DEFAULT_EMPTY_VALUE,
        user_identify_src=DEFAULT_EMPTY_VALUE,
        user_identify_src_username=DEFAULT_EMPTY_VALUE,
        scope_type=DEFAULT_EMPTY_VALUE,
        scope_id=DEFAULT_EMPTY_VALUE,
        **kwargs
    ):
        """
        @type username: str
        @param username: 操作人用户名
        @type request_id: str
        @param request_id: 请求ID
        @type access_type: int
        @param access_type: 访问方式
        @type access_source_ip: str
        @param access_source_ip: 访问来源IP地址
        @type access_user_agent: str
        @param access_user_agent: 访问客户端类型
        @type user_identify_type: int
        @param user_identify_type: 操作人账号类型
        @type user_identify_tenant_id: str
        @param user_identify_tenant_id: 操作人租户ID
        @type user_identify_src: str
        @param user_identify_src: 操作人账号来源
        @type user_identify_src_username: str
        @param user_identify_src_username: 操作人账号
        @type scope_type: str
        @param scope_type: 管理空间类型
        @type scope_id: str
        @param scope_id: 管理空间ID
        @rtype: AuditContext
        """
        self.username = username
        self.request_id = request_id
        self.access_type = access_type
        self.access_source_ip = access_source_ip
        self.access_user_agent = access_user_agent
        self.user_identify_type = user_identify_type
        self.user_identify_tenant_id = user_identify_tenant_id
        self.user_identify_src = user_identify_src
        self.user_identify_src_username = user_identify_src_username
        self.scope_type = scope_type
        self.scope_id = scope_id
        for key, val in kwargs.items():
            setattr(self, key, val)



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - 审计中心 (BlueKing - Audit Center) available.
Copyright (C) 2022 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import warnings

from bk_audit.log.exporters import BaseExporter
from bk_audit.log.formatters import BaseFormatter
from bk_audit.log.queue import AuditEventQueue, BaseQueue


class BkAuditLog(object):
    """
    处理审计日志
    """

    def __init__(self, bk_app_code, bk_app_secret, queue=None):
        """
        @type bk_app_code: str
        @param bk_app_code: App Code
        @type bk_app_secret: str
        @param bk_app_secret: App Secret
        @type queue: BaseQueue
        @param queue: 审计事件队列
        @rtype: BkAuditLog
        """
        self._bk_app_code = bk_app_code
        self._bk_app_secret = bk_app_secret
        self._queue = queue or AuditEventQueue()
        self._formatter = None
        self._sync_exporters = []
        self._delay_exporters = []
        self._exporter_class = []

    def _check_init(self):
        """
        确保必要的参数都已经设置
        """
        assert isinstance(self._queue, BaseQueue), "Queue Invalid"
        assert isinstance(self._formatter, BaseFormatter), "Formatter Invalid"
        assert self._delay_exporters or self._sync_exporters, "Exporter Unset"

    def set_queue_limit(self, limit):
        """
        设置队列长度
        @type limit: int
        @param limit: 审计事件队列最大长度
        """
        self._queue.set_limit(limit)

    def set_formatter(self, formatter):
        """
        设置事件处理器
        @type formatter: BaseFormatter
        @param formatter: 事件处理器
        """
        self._formatter = formatter

    def add_exporter(self, exporter):
        """
        添加事件输出
        @type exporter: BaseExporter
        @param exporter: 事件输出器
        """
        assert isinstance(exporter, BaseExporter), "Exporter Invalid"
        if exporter.__class__ in self._exporter_class:
            warnings.warn("Exporter Duplicate")
        self._exporter_class.append(exporter.__class__)
        if exporter.is_delay:
            self._delay_exporters.append(exporter)
        else:
            self._sync_exporters.append(exporter)

    def add_event(self, **kwargs):
        """
        新增审计事件
        """
        self._check_init()
        event = self._formatter.build_event(**kwargs)
        event.bk_app_code = self._bk_app_code
        # 存在延迟导出的则需要将日志添加到队列
        if self._delay_exporters:
            self._queue.add(event)
        # 同步导出的直接调用
        for exporter in self._sync_exporters:
            exporter.export([event])

    def export_events(self):
        """
        导出队列中的审计事件
        """
        self._check_init()
        events = self._queue.pop_all()
        for exporter in self._delay_exporters:
            exporter.export(events)



# -*- coding: utf-8 -*-
"""
TencentBlueKing is pleased to support the open source community by making
蓝鲸智云 - 审计中心 (BlueKing - Audit Center) available.
Copyright (C) 2022 THL A29 Limited,
a Tencent company. All rights reserved.
Licensed under the MIT License (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at http://opensource.org/licenses/MIT
Unless required by applicable law or agreed to in writing,
software distributed under the License is distributed on
an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
either express or implied. See the License for the
specific language governing permissions and limitations under the License.
We undertake not to change the open source license (MIT license) applicable
to the current version of the project delivered to anyone in the future.
"""

import abc
import datetime
from functools import cached_property
from typing import Optional, TypedDict, Union

from bk_resource import Resource
from bk_resource.base import Empty
from blueapps.core.exceptions import BlueException
from django.utils.module_loading import import_string
from iam import Action as SimpleAction
from iam import Resource as SimpleResourceType
from iam.model.models import Action, ResourceType
from rest_framework import status

from bk_audit.client import BkAudit
from bk_audit.constants.log import DEFAULT_EMPTY_VALUE, DEFAULT_RESULT_CODE
from bk_audit.log.models import (
    AuditAction,
    AuditContext,
    AuditInstance,
    AuditResourceType,
)


# noinspection PyCompatibility
class AuditEvent(TypedDict):
    action: Union[AuditAction, SimpleAction, Action]
    resource_type: Optional[Union[AuditResourceType, SimpleResourceType, ResourceType]]
    instance: Optional[AuditInstance]
    audit_context: Optional[AuditContext]
    event_id: Optional[str]
    event_content: Optional[str]
    start_time: Optional[int]
    end_time: Optional[int]
    result_code: Optional[int]
    result_content: Optional[str]
    extend_data: Optional[dict]


# noinspection PyCompatibility
class AuditMixin(abc.ABC):
    """
    审计集成
    """

    _tmp_context: object
    _audit_event: AuditEvent
    _bk_audit_client_path = "bk_audit.contrib.bk_audit.client.bk_audit_client"
    name: str

    @cached_property
    def _bk_audit_client(self) -> BkAudit:
        return import_string(self._bk_audit_client_path)

    @property
    def audit_action(self) -> Optional[Union[AuditAction, SimpleAction, Action]]:
        """
        操作
        """

        return None

    @property
    def audit_resource_type(self) -> Optional[Union[AuditResourceType, SimpleResourceType, ResourceType]]:
        """
        资源类型
        """

        return None

    def _init_audit_event(self, request_data=None, **kwargs) -> AuditEvent:
        """
        初始化审计事件
        """

        return {
            "action": self.audit_action,
            "resource_type": self.audit_resource_type,
            "instance": None,
            "audit_context": None,
            "event_id": None,
            "event_content": str(self.name),
            "start_time": int(datetime.datetime.now().timestamp() * 1000),
            "end_time": None,
            "result_code": DEFAULT_RESULT_CODE,
            "result_content": DEFAULT_EMPTY_VALUE,
            "extend_data": {},
        }

    def _add_audit_event_error(self, error: Union[BlueException, Exception]) -> None:
        """
        添加异常信息到审计事件
        """

        if hasattr(error, "code") and hasattr(error, "message"):
            self._audit_event.update({"result_code": error.code, "result_content": str(error.message)})
            return
        self._audit_event.update({"result_code": status.HTTP_500_INTERNAL_SERVER_ERROR, "result_content": str(error)})

    def _add_audit_event_response(self, response: any) -> None:
        """
        添加响应到审计事件
        """

        return

    def _add_audit_event_instance(self) -> None:
        """
        添加实例数据到审计事件
        """

        self._audit_event["instance"] = getattr(self._tmp_context, "_audit_instance", None)

    def add_audit_instance_to_context(self, instance: AuditInstance) -> None:
        """
        添加实例数据到上下文
        """

        setattr(self._tmp_context, "_audit_instance", instance)

    def _before_add_event(self) -> None:
        """
        上报前处理审计事件
        """

        self._audit_event["end_time"] = int(datetime.datetime.now().timestamp() * 1000)


# noinspection PyCompatibility
class AuditMixinResource(Resource, AuditMixin, abc.ABC):
    """
    集成审计日志上报的 Resource
    """

    def __init__(self, context=None):
        super().__init__(context=context)
        self._tmp_context = Empty()

    def request(self, request_data=None, **kwargs):
        """
        调用Resource
        """

        # 判定是否需要上报审计事件
        if not self._enabled_audit_report():
            return super().request(request_data=request_data, **kwargs)

        # 初始化审计事件
        self._audit_event = self._init_audit_event(request_data=request_data, **kwargs)

        try:
            # 调用Resource
            response_data = super().request(request_data=request_data, **kwargs)
            # 添加响应内容到审计事件
            self._add_audit_event_response(response=response_data)
            return response_data
        except Exception as err:
            # 添加异常到审计事件
            self._add_audit_event_error(error=err)
            raise err
        finally:
            # 添加实例数据到审计事件
            self._add_audit_event_instance()
            # 上报前处理审计事件
            self._before_add_event()
            self._bk_audit_client.add_event(**self._audit_event)

    def _enabled_audit_report(self) -> bool:
        """
        判定是否需要上报审计事件
        """

        return self.audit_action is not None
